function [t,y] = ExponentialRK2(f,df,y0,T,h,alpha)
% this code solves the differential equation y'=F(y) and Jacobi DF using a
% 2-stage Exponential Runge-Kutta method.
%

 
  
  if nargin < 6;
    alpha = 1;
  end
  
% Initialize variables
  N = ceil((T(2)-T(1))/h); % number of steps
  h = (T(2)-T(1))/N; % adjust step size to fit interval length
  d = size(y0,1); % dimension of solution
  t = linspace(T(1),T(2),N+1); % time grid
  y = nan(d,N+1); % solution
  y(:,1) = y0; % set initial value


% Compute solution
  for j=1:N
    y(:,j+1) = ExponentialRK2Step(f,df,y(:,j),t,h,alpha);
  end

end

function y = ExponentialRK2Step(f,df,y0,t,h,alpha)

  c  = 0.5;
  a21 = alpha;
  c21 = 0.75*alpha^2 - alpha;

  b = nan(size(y0));
  b(2) = 1/(3*alpha^2);
  b(1) = 1 - b(2);

  DF = df(t,y0);

  phi = ( expm(c*h*DF)-eye(size(DF)) )/(c*h*DF);

  u = nan(size(y0));
  k = nan(size(y0));
  
  u(:,1) = y0;
  k(:,1) = phi*f(t,u(:,1));
  
  u(:,2) = y0 + h*a21*k(:,1);
  k(:,2) = phi*( f(t,u(:,2)) + h*DF*c21*k(:,1) );

  y = y0 + h*(b(1)*k(:,1) + b(2)*k(:,2));

end
